/*
 * Created on 23.09.2004
 */
package casino;

import org.apache.log4j.*;
import org.aspectj.lang.*;
import org.aspectj.lang.reflect.*;

/**
 * @author oliver
 * @since 23.09.2004
 */
public aspect LogAspect {
    
    private static Logger log = Logger.getLogger(LogAspect.class); 
    
    pointcut executeMethods() :
        execution(public * *(..)) && !(within(LogAspect));
    
    pointcut callConstructors() : 
        call(public bank.*.new(..)) && !(within(LogAspect));
    
    pointcut logPoints() :
        executeMethods() || callConstructors();
    
    before() : logPoints() {
        SourceLocation loc = thisJoinPoint.getSourceLocation();
        log.debug("(" + loc + ") " + thisJoinPoint.toShortString());
    }
    
    
    
    /*
     * Automatische Protokollierung aller abgefangenen Exceptions
     */
    
    pointcut handleExceptions() :
        handler(Exception+);
    
    before() : handleExceptions() {
        Signature sig = thisJoinPoint.getSignature();
        log.warn(sig + " in " + thisJoinPoint.getThis());
    }
    
    after() : call(public RuntimeException+.new(..)) {
        Signature sig = thisJoinPoint.getSignature();
        log.error(sig.getDeclaringTypeName() + " aufgetreten in "
                + thisJoinPoint.getSourceLocation());
    }
    
    
    
    /**
     * Buchfuehrung ueber saemtliche Methodenaufrufe von store() und load()
     */
    before() : call(public * *..*(..)) &&
            (withincode(public void Konto.store())
            || withincode(public void Konto.load())) {
        log.debug("store/load: " + thisJoinPoint.toShortString());
    }
    
    
    
    /**
     * Kontostand-Ueberwachung
     */
    after(int x) : set(private int Konto.kontostand) && args(x) {
        log.info("Kontostand aendert sich zu " + x);
    }

}
