/**
 * Ein einfaches Beispiel fuer den Annotation-basierten Codierungsstil:
 * Es werden die Rotphasen einer Ampel protokolliert.
 */
package verkehrs;

import java.text.*;
import java.util.Date;
import org.aspectj.lang.annotation.*;

@Aspect
public class RotlichtAspect {

    private boolean isRed = false;

    private long start = 0;

    @After("set(int AmpelModel.state) && args(x)")
    public void watchAmpel(int x) {
        if (isRed) {
            if (x != AmpelModel.RED) {
                isRed = false;
                long end = System.currentTimeMillis();
                printRotphase(start, end);
            }
        } else {
            if (x == AmpelModel.RED) {
                isRed = true;
                start = System.currentTimeMillis();
            }
        }
    }

    public void printRotphase(long start, long end) {
        DateFormat df = new SimpleDateFormat("HH:mm:ss.SSS");
        System.out.println("Rotphase:" + " von " + df.format(new Date(start))
                + " bis " + df.format(new Date(end)) + " (" + (end - start)
                + " ms)");
    }

}



/*
 * Hier zum Vergleich die Code-basierte Schreibweise:

public aspect RotlichtAspect {
	
	private boolean isRed = false;
	private long start = 0;

	after(int x) : set(int AmpelModel.state) && args(x) {
		if (isRed) {
			if (x != AmpelModel.RED) {
				isRed = false;
				long end = System.currentTimeMillis();
				printRotphase(start, end);
			}
		} else {
			if (x == AmpelModel.RED) {
				isRed = true;
				start = System.currentTimeMillis();
			}
		}
	}
	
	public void printRotphase(long start, long end) {
		DateFormat df = new SimpleDateFormat("HH:mm:ss.SSS");
		System.out.println("Rotphase:"
				+ " von " + df.format(new Date(start))
				+ " bis " + df.format(new Date(end))
				+ " (" + (end - start) + " ms)");
	}
	
}

 */