/*
 * Created on 03.01.2005
 *
 * $Id$
 */
package casino;

import java.io.*;

import casino.player.*;

/**
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 03.01.2005
 * @version $revision$
 */
public class Croupier implements Runnable {
    
    /** Croupier kann nur 1 Spieler bedienen - das reicht vorerst */
    private RoulettePlayer player = null;
    
    /** Stammkapital erhoeht, um auch laenger spielen zu koennen */
    private Konto spielbank = new Konto("Spielbank", 5000);
    
    /** und natuerlich braucht er noch einen Roulette-Tisch */
    private Roulette roulette = new Roulette();
    
    /**
     * Kleine Begruessung ausgeben und nach dem Namen fragen.
     * @return
     */
    public String welcome() {
        Reader istreamReader = new InputStreamReader(System.in);
        BufferedReader reader = new BufferedReader(istreamReader);
        System.out.print("Ihren Namen bitte: ");
        try {
            return reader.readLine();
        } catch (IOException unexpected) {
            unexpected.printStackTrace();
            return "Nobody";
        }        
    }
    
    /**
     * Der Spieler muss sich beim Croupier vorstellen.
     * @param name
     */
    public void login(String name) {
        Konto konto = new Konto(name, 5000);
        this.login(new Player(name, konto));
    }
    
    /**
     * Der Spieler muss sich beim Croupier anmelden.
     * @param john
     */
    public void login(Player john) {
        this.player = john;
        System.out.println("Guten Tag, " + john
                + " - Ihr Kontostand betraegt "
                + john.getKontostand() + " Chips");
    }
    
    /**
     * Welcher Spieler sitzt den am Tisch?
     * @return
     */
    public RoulettePlayer getPlayer() {
        return this.player;
    }
    
    /**
     * kleines Probespiel gefaellig?
     */
    public void playTheGame() {
        for (int i = 0; i < 100; i++) {
            Einsatz einsatz = player.getEinsatz();
            spielbank.einzahlen(einsatz.getBetrag());
            //System.out.print("Rien ne va plus...");
            roulette.werfeKugel();
            //System.out.println(" " + roulette);
            Einsatz gewinn = getGewinn(einsatz);
            player.setGewinn(gewinn);
            spielbank.abheben(gewinn.getBetrag());
        }
        System.out.print("Kontostand " + player +": " + player.getKontostand());
        System.out.println(" / Spielbank: " + spielbank.abfragen());
    }
    
    /**
     * and go, go, go...
     */
    public void run() {
        playTheGame();
    }
    
    /**
     * Gewinn-Ermittlung mit Hilfe des Biet-Einsatzes.
     * @param einsatz
     * @return
     */
    private Einsatz getGewinn(Einsatz einsatz) {
        Einsatz abgerechnet = new Einsatz();
        if (roulette.isPair()) {
            abgerechnet.setPair(einsatz.getPair() * 2);
        }
        if (roulette.isImpair()) {
            abgerechnet.setImpair(einsatz.getImpair() * 2);
        }
        if (roulette.isManque()) {
            abgerechnet.setManque(einsatz.getManque() * 2);
        }
        if (roulette.isPasse()) {
            abgerechnet.setPasse(einsatz.getPasse() * 2);
        }
        int wurf = roulette.getWurf();
        int chips = einsatz.getZahl(wurf) * 36;
        abgerechnet.setZahl(wurf, chips);
        return abgerechnet;
    }

    /**
     * Hauptprogramm, um unsere kleine Simulation zu starten
     * 
     * @param args  keine
     * @author <a href="mailto:boehm@javatux.de">oliver</a>
     * @since  07.01.2005
     */
    public static void main(String[] args) {
        System.out.println("Casino 'Runnable' online");
        Croupier croupier1 = new Croupier();
        Croupier croupier2 = new Croupier();
        Player player1 = new Player("Asterix");
        Player player2 = new DoubleBetPlayer("Obelix");
        croupier1.login(player1);
        croupier2.login(player2);
        Thread thread1 = new Thread(croupier1);
        Thread thread2 = new Thread(croupier2);
        thread1.start();
        thread2.start();
        System.out.println("beide Croupiers arbeiten jetzt...");
    }

}
